/* Copyright 2021 Aristocratos (jakob@qvantnet.com)

   Licensed under the Apache License, Version 2.0 (the "License");
   you may not use this file except in compliance with the License.
   You may obtain a copy of the License at

       http://www.apache.org/licenses/LICENSE-2.0

   Unless required by applicable law or agreed to in writing, software
   distributed under the License is distributed on an "AS IS" BASIS,
   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
   See the License for the specific language governing permissions and
   limitations under the License.

indent = tab
tab-size = 4
*/

#pragma once

#include <string>
#include <vector>
#include <robin_hood.h>
#include <filesystem>

using std::string, std::vector, robin_hood::unordered_flat_map;

//* Functions and variables for reading and writing the btop config file
namespace Config {

	extern std::filesystem::path conf_dir;
	extern std::filesystem::path conf_file;

	extern unordered_flat_map<string, string> strings;
	extern unordered_flat_map<string, bool> bools;
	extern unordered_flat_map<string, int> ints;

	const vector<string> valid_graph_symbols = { "braille", "block", "tty" };

	extern vector<string> current_boxes;

	//* Check if string only contains space seperated valid names for boxes
	bool check_boxes(string boxes);

	//* Return bool for config key <name>
	inline const bool& getB(const string& name) { return bools.at(name); }

	//* Return integer for config key <name>
	inline const int& getI(const string& name) { return ints.at(name); }

	//* Return string for config key <name>
	inline const string& getS(const string& name) { return strings.at(name); }

	//* Set config key <name> to bool <value>
	void set(string name, bool value);

	//* Set config key <name> to int <value>
	void set(string name, int value);

	//* Set config key <name> to string <value>
	void set(string name, string value);

	//* Flip config key bool <name>
	void flip(string name);

	//* Lock config and cache changes until unlocked
	void lock();

	//* Unlock config and write any cached values to config
	void unlock();

	//* Load the config file from disk
	void load(std::filesystem::path conf_file, vector<string>& load_errors);

	//* Write the config file to disk
	void write();
}